/*
   Версия: 22E от 15.07.2020
   Что нового: Исправлен баг со сбрасыванием накопленной энергии после обновления облака
   Добавлена возможность использовать датчики температуры DS18B20 - 2 шт, при этом нужно помнить, что на
   сцене должны иметься резервные переменные
   Железо: wemos d1 mini + RS232 to TTL converter (MAX3232)
   Подключение: TX ESP <-> TX TTL
                RX ESP <-> RX TTL
                GND ESP <-> GND TTL
                3.3V ESP <-> VCC TTL напряжение в зависимости от чипа TTL конвертора
                3,3V - MAX3232
                5V - MAX232
                D5 <-> Датчик температуры 1 (DS18B20)
                D6 <-> Датчик температуры 2 (DS18B20)
   Мониторинг параметров и статуса инвертора. Передача на сервис аналитики http://open-monitoring.online
   Nikolay Semerkov, https://nsk-electro.ru/
   Для связи с автором: semerkov@mail.ru
*/

// Строки требующие редактирования в комментариях отмечены (!)

#include <ESP8266WiFi.h>
#include <string.h>
#include <OneWire.h>

//*-- Индивидуальные настройки
const char* ssid     = "******";                 // Название точки доступа     (!)
const char* password = "******";                 // Пароль                     (!)

//*-- Параметры полученные при регистрации на http://open-monitoring.online
const char* host = "open-monitoring.online";     // Адрес сервера (не менять)
const char* streamId   = "***";                  // ID                         (!)
const char* privateKey = "******";               // Код доступа                (!)
unsigned long send_interval = 61000;             // Периодичность отправки пакетов на сервер
                                                 // (не менее 60 сек)
boolean sendTemp = false;                        // Использовать датчики температуры "false" - нет, "true" - да

// Служебные перемнные
String Inv_param = "";     //Строка с текущими параметрами инвертора
String Inv_status = "";    //Строка с текущим статусом инвертора

// Текущие параметры инвертора
char U_grid [6] ;        // Напряжение входной сети
char F_grid [6] ;        // Частота входной сети
char U_out [6] ;         // Выходное напряжение
int S_out ;              // Выходня мощность полная (ВА)
int P_out ;              // Выходная мощность активная (Вт)
int Load ;               // Нагрузка инвертора, (%)
char U_bat [6] ;         // Напряжение АКБ
double U_bat_dec;        // Напряжение АКБ в десятичной форме
int I_charge ;           // Ток заряда АКБ
int C_bat ;              // Уровень заряда АКБ (%)
int T_inv ;              // Температура инвертора (NTC)
int I_pv_bat ;           // Ток заряда АКБ от СБ
char U_pv [6] ;          // Напряжение СБ
int I_discharge ;        // Ток разряда АКБ

float Psolar = 0 ;       // Мощность снимаемая с панелей
float Power_sum = 0 ;    // Переменная, хранящая энергию до ее отправки на сервер и
                         // получения подтверждения о ее принятии
float Power_count = 0 ;  // Количество энергии за последнюю итерацию
unsigned long delta_T = 0 ;
unsigned long preT = 0 ;

unsigned long previousMillis = 0;
boolean flag_1Send = true; // Флаг первой посылки

// Команды запроса в инвертор
char RECUEST_PARAM[8] = {81, 80, 73, 71, 83, 183, 169, 13};  // Запрос текущих параметров
char RECUEST_STATUS[7] = {81, 77, 79, 68, 73, 193, 13};      // Запрос статуса устройства

OneWire sens1(14);    // № пина подключения датч. тем-ры 1 (D5 - обознач. на плате ESP)
OneWire sens2(12);    // № пина подключения датч. тем-ры 2 (D6 - обознач. на плате ESP)

OneWire * psens1 = &sens1;
OneWire * psens2 = &sens2;

//*---------------- INICIALIZATION ------------------*//
void setup() {
  Serial.begin(2400); // Скорость UART для работы с инвертором
  delay(10);

  // We start by connecting to a WiFi network

  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}


//*------------------ MAIN CYCLE --------------------*//
void loop() {

  request_param(); // Запрашиваем параметры инвертора
  calc_Watt();     // Рссчитываем выработанную энергию

  unsigned long currentMillis = millis();
  if ( currentMillis - previousMillis >= send_interval || flag_1Send == true ) {
    previousMillis = currentMillis;
    flag_1Send = false; // Опускаем флаг первой отправки

    request_status();  // Запрос статуса
    data_send();       // Отправка данных на сервер
  }
}
